Package: FamAgg
Type: Package
Title: Pedigree Analysis and Familial Aggregation
Version: 1.40.0
Authors@R: c(person(given = "Johannes", family = "Rainer",
	   email = "johannes.rainer@eurac.edu",
	   role = c("aut", "cre"),
	   comment = c(ORCID = "0000-0002-6977-7147")),
	   person(given = "Daniel", family = "Taliun",
	   role = "ctb"),
	   person(given = "Christian", family = "Weichenberger",
	   email = "christian.weichenberger@eurac.edu",
	   role = "ctb",
	   comment = c(ORCID = "0000-0002-2176-0274"))
	  )
Author: J. Rainer, D. Taliun, C.X. Weichenberger
Maintainer: Johannes Rainer <johannes.rainer@eurac.edu>
URL: https://github.com/EuracBiomedicalResearch/FamAgg
BugReports: https://github.com/EuracBiomedicalResearch/FamAgg/issues
Imports: gap (>= 1.1-17), Matrix, BiocGenerics, utils, survey
Depends: methods, kinship2, igraph
Suggests: BiocStyle, knitr, RUnit, rmarkdown
VignetteBuilder: knitr
Description: Framework providing basic pedigree analysis and plotting
        utilities as well as a variety of methods to evaluate familial
        aggregation of traits in large pedigrees.
Collate: Classes.R Generics.R Deprecated.R Constructors.R
        Methods-FAData.R Methods-FAKinGroupResults.R
        Methods-FAKinSumResults.R Methods-FAGenIndexResults.R
        Methods-FAIncidenceRateResults.R
        Methods-FAStdIncidenceRateResults.R Methods-FABinTestResults.R
        utils.R matched-controls.R Methods.R plotting-functions.R
        import-export.R zzz.R
biocViews: Genetics
License: MIT + file LICENSE
Copyright: Haplopainter1.043.pl (http://haplopainter.sourceforge.net)
        Copyright (c) 2004-2008 Holger Thiele
NeedsCompilation: no
RoxygenNote: 7.1.1
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libuv1-dev
        libxml2-dev libssl-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:42:15 UTC
RemoteUrl: https://github.com/bioc/FamAgg
RemoteRef: RELEASE_3_23
RemoteSha: e5d4b24ca16b609b6186b63d13b01d1ad2590cae
Packaged: 2026-04-29 21:14:32 UTC; root
Built: R 4.6.0; ; 2026-04-29 22:12:21 UTC; unix
