## Namespace
import(utils)
import(methods)
importFrom("stats", "density", "approxfun")
## import(BiocGenerics)
##import(igraph)
importFrom("stats", "binom.test", "dbinom", "pbinom")
importFrom(igraph, graph, distances, shortest_paths, V, induced_subgraph, clusters)
importClassesFrom(Matrix, "dsCMatrix")
importMethodsFrom(Matrix, as.matrix, colSums, colMeans, "[", which)
importFrom(BiocGenerics, "density")
importFrom(gap, pfc.sim)
import(kinship2)
importFrom(survey, stratsample)

importFrom("graphics", "abline", "hist", "legend", "par", "plot",
           "points", "strheight", "strwidth", "text")
importFrom("grDevices", "dev.off", "pdf", "png")
importFrom("stats", "aggregate", "p.adjust", "p.adjust.methods")
##importFrom(kinship2, kinship, pedigree, plot.pedigree)
##importClassesFrom(graph, graphNEL)
##importFrom(graph, graphNEL)
## importMethodsFrom(graph, leaves, isConnected, edges, nodes, isDirected,
##                   removeNode)
## importFrom(DBI, dbDriver)
## importClassesFrom(AnnotationHub, AnnotationHub)
## importMethodsFrom(AnnotationHub, query, mcols)

## exported functions
export("FAData", "connectedSubgraph", "doPlotPed", "estimateTimeAtRisk",
       "factor2matrix", "ped2graph", "sliceAge", "subPedigree", "switchPlotfun")
## exported classes
exportClasses("FAData", "FAProbResults", "FAKinGroupResults",
              "FAKinSumResults", "FAGenIndexResults", "FAIncidenceRateResults",
              "FAStdIncidenceRateResults", "FABinTestResults")
## exported methods
exportMethods("$", "[", "affectedIndividuals", "affectedKinshipGroups", "age",
              "age<-", "buildPed", "cliques", "cliques<-", "cliqueAndTrait",
              "export", "family", "kinship", "lambda", "pedigree", "pedigree<-",
              "pedigreeSize", "phenotypedIndividuals", "removeSingletons",
              "plotPed", "plotRes", "resultForId", "shareKinship", "result",
              "trait", "trait<-", "traitByClique", "kinship", "timeAtRisk",
              "timeAtRisk<-",
              "timeInStrata")
## basic pedigree utils
exportMethods("countGenerations", "estimateGenerations", "generationsFrom",
              "getAncestors", "getChildren", "getSiblings", "getMissingMate",
              "getCommonAncestor", "findFounders", "getFounders",
              "getSingletons")
## matched control methods
exportMethods("getAll", "getExternalMatched", "getGenerationMatched",
              "getGenerationSexMatched", "getSexMatched")

## analysis methods...
exportMethods("familialIncidenceRate", "familialIncidenceRateTest", "fsir",
              "fsirTest", "genealogicalIndexTest", "kinshipSumTest",
              "kinshipGroupTest", "probabilityTest", "runSimulation")
export("binomialTest")
export("kinshipPairs")
