[![Years in Bioconductor](http://www.bioconductor.org/shields/years-in-bioc/FamAgg.svg)](http://www.bioconductor.org/packages/release/bioc/html/FamAgg.html)
[![Bioconductor release build status](http://www.bioconductor.org/shields/build/release/bioc/FamAgg.svg)](http://www.bioconductor.org/packages/release/bioc/html/FamAgg.html)
[![Bioconductor devel build status](http://www.bioconductor.org/shields/build/devel/bioc/FamAgg.svg)](http://www.bioconductor.org/checkResults/devel/bioc-LATEST/FamAgg.html)
[![build status](https://github.com/EuracBiomedicalResearch/FamAgg/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/EuracBiomedicalResearch/FamAgg/actions?query=workflow%3AR-CMD-check-bioc)
[![codecov.io](https://codecov.io/github/EuracBiomedicalResearch/FamAgg/coverage.svg?branch=master)](https://codecov.io/github/EuracBiomedicalResearch/FamAgg?branch=master)

# FamAgg: Pedigree Analysis and Familial Aggregation

<img align = "right" src="https://raw.githubusercontent.com/Bioconductor/BiocStickers/master/FamAgg/FamAgg.png" height="200">

Framework providing basic pedigree analysis and plotting utilities as well as a
variety of methods to evaluate familial aggregation of traits in large
pedigrees.
